import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { EventScheduleRepository } from './event-schedule.repository';
import { EventScheduleEntity } from './entities/event-schedule.entity';
import { CreateEventScheduleDto } from './dto/createEventSchedule.dto';
import { UsersRepository } from '../users/user.repository';
import { UpdateEventScheduleDto } from './dto/updateEventSchedule.dto';
import { MailSenderService } from '../mail-sender/mail-sender.service';
import { GuestInviteMailDto } from '../mail-sender/dto/guest-invite-mail.dto';
import { CreateNotificationDto } from '../notification/dto/create-notification.dto';
import { notificationType } from 'src/enum-type/notification.enum';
import { NotificationRepository } from '../notification/notification.repository';
import { EventScheduleGuestRepository } from './event-schedule-guest.repository';
import { EventScheduleGuestEntity } from './entities/event-schedule-guest.entity';

@Injectable()
export class EventScheduleService {
    constructor(
        @InjectRepository(UsersRepository)
        private readonly usersRepository: UsersRepository,
        @InjectRepository(EventScheduleRepository)
        private readonly eventScheduleRepository: EventScheduleRepository,
        private readonly servMail: MailSenderService,
        private readonly eventScheduleGuestRepository: EventScheduleGuestRepository,
        @InjectRepository(NotificationRepository)
        private readonly notificationRepository: NotificationRepository
    ) {
    }

    async create(dto: CreateEventScheduleDto, user: any): Promise<EventScheduleEntity | Error> {
        try {
            const { guestNameEmail, transactionId } = dto;

            const emailExists = guestNameEmail.some(obj => obj['guestEmail'] === user.email)
            if (emailExists) {
                throw new HttpException(
                    "You can't schedule an event with you!",
                    HttpStatus.BAD_REQUEST
                );
            }

            const data = await this.eventScheduleRepository.createEntity(dto);
            if (!data) {
                throw new HttpException(
                    Constants.CREATE_FAILED,
                    HttpStatus.BAD_REQUEST
                );
            }

            for (const element of dto.guestNameEmail) {
                const isGuestExist = await this.usersRepository.findOne({
                    email: element['guestEmail'],
                    isDeleted: false
                });

                const createGuestObj = {
                    eventScheduleId: data?.id,
                    guestEmail: element["guestEmail"],
                    invitedUserId:isGuestExist ? isGuestExist.id : "",
                    invitedBy: user.id,
                    guestName: element['guestName']
                }

                const dataGuest = await this.eventScheduleGuestRepository.createEntity(createGuestObj);
                if (!dataGuest) {
                    throw new HttpException(
                        Constants.CREATE_FAILED,
                        HttpStatus.BAD_REQUEST
                    );
                }

                // create notification
                const notificationDto: CreateNotificationDto = {
                    userId: isGuestExist ? isGuestExist.id : null,
                    type: notificationType.SCHEDULE,
                    scheduleId: (data as EventScheduleEntity)?.id,
                    transactionId
                }
                this.notificationRepository.createEntity(notificationDto);

                if (!isGuestExist?.id) {
                    //invite guest email
                    let mailDto = {
                        to: [element["guestEmail"]],
                        from: process.env.EMAIL_SENDER_MAIL,
                        subject: "Transaction Bee Invitation",
                        guestName: element["guestName"],
                    };

                    await this.servMail.guestInvitationMail(mailDto as GuestInviteMailDto);
                }                
            }

            return ResponseUtils.successResponseHandler(201, "Data Created Successfully", "data", data);
        } catch (e) {
            console.log(e);
            return e;
        }
    }

    async findAll(user: any): Promise<EventScheduleEntity[] | Error> {
        const data = await this.eventScheduleRepository.findByFilterQuery({ invitedBy: user.id });
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data Fetched Successfully", "data", data);
    }

    async findOne(id: string, user: any): Promise<EventScheduleEntity[] | Error> {
        const data: any = await this.eventScheduleRepository.findByFilterQuery({ invitedBy: user.id });
        if (data.length <= 0) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data Fetched Successfully", "data", data);
    }

    async update(id: string, dto: UpdateEventScheduleDto, user: any): Promise<EventScheduleEntity | Error> {
        const { guestNameEmail } = dto;
        const emailExists = guestNameEmail.some(obj => obj['guestEmail'] === user.email)
            if (emailExists) {
                throw new HttpException(
                    "You can't schedule an event with you!",
                    HttpStatus.BAD_REQUEST
                );
            }
        const updatedInvite: any = await this.eventScheduleRepository.updateEntity(id, dto);
        if (!updatedInvite) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        await this.eventScheduleGuestRepository.deleteByScheduleId(updatedInvite.id)
        for (const element of dto.guestNameEmail) {
            const isGuestExist = await this.usersRepository.findOne({
                email: element['guestEmail'],
                isDeleted: false
            });

            const createGuestObj = {
                eventScheduleId: updatedInvite?.id,
                guestEmail: element["guestEmail"],
                invitedUserId:isGuestExist ? isGuestExist.id : "",
                invitedBy: user.id,
                guestName: element['guestName']
            }

            const dataGuest = await this.eventScheduleGuestRepository.createEntity(createGuestObj);
            if (!dataGuest) {
                throw new HttpException(
                    Constants.CREATE_FAILED,
                    HttpStatus.BAD_REQUEST
                );
            }

            // create notification
            const notificationDto: CreateNotificationDto = {
                userId: isGuestExist ? isGuestExist.id : null,
                type: notificationType.SCHEDULE,
                scheduleId: (updatedInvite as EventScheduleEntity)?.id
            }
            this.notificationRepository.createEntity(notificationDto);

            if (!isGuestExist?.id) {
                //invite guest email
                let mailDto = {
                    to: [element["guestEmail"]],
                    from: process.env.EMAIL_SENDER_MAIL,
                    subject: "Transaction Bee Invitation",
                    guestName: element["guestName"],
                };

                await this.servMail.guestInvitationMail(mailDto as GuestInviteMailDto);
            }
        }
        return ResponseUtils.successResponseHandler(200, "Data Updated Successfully", "data", updatedInvite);
    }

    async remove(id: string, user: any): Promise<boolean | Error> {
        const data = await this.eventScheduleRepository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                `Failed to delete invitation`,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data Deleted Successfully");
    }
}
