import { Injectable } from '@nestjs/common';
import { SingleMessageDto } from './dto/single-message.dto';
import { GroupMessageDto } from './dto/group-message.dto';
import { InjectRepository } from "@nestjs/typeorm";
import { MessageRepository } from "./message.repository";
import { OneToOneMessageDto } from "./dto/one-to-one-message.dto";
import { AllGroupMessageDto } from "./dto/all-group-message.dto";
import { NotificationMessageDto } from "./dto/notification-message.dto";
import { ResponseUtils } from "src/utils/response.utils";
import { FileUploadDto } from "./dto/file-upload.dto";
import { FileUploadsService } from "../file-uploads/file-uploads.service";

@Injectable()
export class MessageService {
    private fileChunks: { [fieldname: string]: Buffer[] } = {}; // Store received file chunks

    constructor(
        @InjectRepository(MessageRepository) private readonly repository: MessageRepository,
        private readonly fileService: FileUploadsService,
    ) { }

    async createSingleMessage(dto: SingleMessageDto) {
        return await this.repository.createEntity(dto);
    }

    async findAllOneToOne(dto: OneToOneMessageDto) {
        return await this.repository.findAllOneToOne(dto);
    }

    async createGroupMessage(dto: GroupMessageDto) {
        return await this.repository.createEntityForGroup(dto);
    }

    async findAllGroupMessage(dto: AllGroupMessageDto) {
        return await this.repository.findAllGroupMessage(dto);
    }

    async findNotificationsMessageByUserId(dto: NotificationMessageDto) {
        const data = await this.repository.findNotificationsMessageByUserId(dto);
        return ResponseUtils.successResponseHandler(200, "Data Fetched Successfully", "data", data);
    }

    async handleFileChunk(dto: FileUploadDto) {
        const { fieldname, buffer } = dto.file;

        // Check if the fieldname already exists in the fileChunks
        if (!this.fileChunks[fieldname]) {
            this.fileChunks[fieldname] = [];
        }

        // Store the received chunk in the array
        this.fileChunks[fieldname].push(buffer);

        if (dto.chunkCount == this.fileChunks[fieldname].length) {
            const data = this.assembleFile(fieldname);
            const rootPath = 'uploads'
            const uploadPath = 'message';
            const filePath = await this.fileService.saveFileToUploadsFolder(data, dto.file.originalname, rootPath, uploadPath);
            return {
                percent: 100,
                filePath
            };
        }

        const chunkCount = dto.chunkCount;
        const uploadCount = this.fileChunks[fieldname].length;

        return {
            percent: uploadCount / chunkCount * 100,
            filePath: null
        };
    }

    private assembleFile(fieldname: string) {
        // Check if all chunks for the specified fieldname have been received
        if (this.fileChunks[fieldname] && this.fileChunks[fieldname].length > 0) {
            // Concatenate all chunks to assemble the complete file
            const completeFile = Buffer.concat(this.fileChunks[fieldname]);

            // Clear the stored chunks for this fieldname
            delete this.fileChunks[fieldname];

            return completeFile;
        }

        // Return null if no chunks have been received yet
        return null;
    }
}
