import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { TransactionRepository } from './transaction.repository';
import { CreateTransactionDto } from './dto/create-transaction.dto';
import { UpdateTransactionDto } from './dto/update-transaction.dto';
import { TransactionEntity } from './entity/transaction.entity';
import { PartyRepository } from '../party/party.repository';
import { PaginationDto } from './dto/create-pagination.dto';
import { TransactionAddressDetailsRepository } from '../transaction-address-details/transaction-address-details.repository';
import { TransactionTimelineDetailsRepository } from '../transaction-timeline-details/transaction-timeline-details.repository';
import { LogsRepository } from '../transaction-logs/logs.repository';
import { TransactionNotesRepository } from "../transaction-notes/transaction-notes.repository";
import { EventScheduleRepository } from '../event-schedule/event-schedule.repository';
import { EventScheduleGuestRepository } from '../event-schedule/event-schedule-guest.repository';
import { TransactionStatus } from './enum/transaction-status.enum';
import { TransactionDocumentsRepository } from '../transaction-documents/transaction-documents.repository';


@Injectable()
export class TransactionService {
    constructor(
        @InjectRepository(TransactionRepository)
        private readonly repository: TransactionRepository,
        @InjectRepository(PartyRepository)
        private readonly partyRepository: PartyRepository,
        @InjectRepository(TransactionAddressDetailsRepository)
        private readonly transactionAddressDetailsRepository: TransactionAddressDetailsRepository,
        @InjectRepository(TransactionTimelineDetailsRepository)
        private readonly transactionTimelineDetailsRepository: TransactionTimelineDetailsRepository,
        @InjectRepository(TransactionNotesRepository)
        private readonly transactionNotesRepository: TransactionNotesRepository,
        @InjectRepository(LogsRepository)
        private readonly logsRepository: LogsRepository,
        @InjectRepository(EventScheduleRepository)
        private readonly eventScheduleRepository: EventScheduleRepository,
        @InjectRepository(EventScheduleGuestRepository)
        private readonly eventScheduleGuestRepository: EventScheduleGuestRepository,
        @InjectRepository(TransactionDocumentsRepository)
        private readonly transactionDocumentsRepository: TransactionDocumentsRepository,
    ) {
    }

    async create(dto: CreateTransactionDto, user: any): Promise<TransactionEntity | Error> {
        try {
            dto.userId = user.id;
            dto.userName = user.firstName + " " + user.lastName;
            const data: any = await this.repository.createEntity(dto);
            if (!data) {
                throw new HttpException(
                    Constants.CREATE_FAILED,
                    HttpStatus.BAD_REQUEST
                );
            } else {
                const createPartyDto = {
                    transactionId: data?.id,
                    userId: dto.userId,
                    userName: dto.userName
                }
                await this.partyRepository.createEntity(createPartyDto);
                await this.transactionAddressDetailsRepository.createEntity({
                    transactionId: data?.id,
                    buyerName: JSON.stringify([dto.buyerName]),
                    sellerName: JSON.stringify([dto.sellerName]),
                    buyerAddress: dto.agent == 'buyer' ? dto.streetAddress : dto.agent == 'both' ? dto.streetAddress : '',
                    sellerAddress: dto.agent == 'listing' ? dto.streetAddress : dto.agent == 'both' ? dto.streetAddress : '',
                });
                await this.transactionTimelineDetailsRepository.createEntity({
                    transactionId: data?.id,
                    closingDate: dto.closingDate
                });
                await this.transactionNotesRepository.createEntity({
                    transactionId: data?.id,
                    userId: dto.userId,
                    message: dto.additionalNotes,
                    file: null,
                    fileName: null,
                    userName: dto.userName
                });
                const eventObj = {
                    eventName: "Closing Date Fixed",
                    startTime: dto.closingDate,
                    endTime: dto.closingDate,
                    description: "Transaction Created",
                    transactionId: data?.id,
                    isModified: true
                }
                const eventData = await this.eventScheduleRepository.createTransactionEventEntity(eventObj);
                if (!eventData) {
                    throw new HttpException(
                        Constants.CREATE_FAILED,
                        HttpStatus.BAD_REQUEST
                    );
                }

                const createGuestObj = {
                    eventScheduleId: eventData?.id,
                    guestEmail: null,
                    invitedUserId: null,
                    invitedBy: user.id,
                    guestName: null
                }

                const dataGuest = await this.eventScheduleGuestRepository.createEntity(createGuestObj);
                if (!dataGuest) {
                    throw new HttpException(
                        Constants.CREATE_FAILED,
                        HttpStatus.BAD_REQUEST
                    );
                }

                // create logs
                const createLogsDto = {
                    userId: dto.userId,
                    address: dto.streetAddress,
                    clientName: dto.clientName,
                    informationChnage: "Transaction created",
                    reasonForChanging: null,
                    status: dto.status
                }
                await this.logsRepository.createEntity(createLogsDto);

                return ResponseUtils.successResponseHandler(201, "Data Created Successfully", "data", data);
            }
        } catch (error) {
            console.log(error);
            return error;
        }
    }

    async findAll(user: any): Promise<TransactionEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({ userId: user.id });
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data Fetched Successfully", "data", data);
    }

    async getTransactionReport(user: any): Promise<TransactionEntity[] | Error> {
        const transactionIds = await this.partyRepository.find({
            where: { userId: user.id, isDeleted: false },
            select: ["transactionId"]
        });

        const ids = transactionIds?.map(x => x.transactionId )

        const data = await this.repository.transactionReportQuery(ids);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data Fetched Successfully", "data", data);
    }

    // async findAll(user: any, paginationDto: PaginationDto): Promise<Partial<TransactionEntity[]> | Error> {
    //     const { page, limit } = paginationDto;
    //     const offset = (page - 1) * limit;


    //     const data = await this.repository.findByFilterQuery({ userId: user.id }, offset, limit);

    //     if (!data) {
    //         throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
    //     }

    //     const totalCount = await this.repository.count({ userId: user.id });

    //     return ResponseUtils.successResponseHandler(200, "Data Fetched Successfully", "data", data, { page, limit, totalCount });
    // }

    async filterTransaction(user: any, paginationDto: PaginationDto): Promise<any | Error> {
        const dataResponse = await this.repository.filterTransaction(user.id, paginationDto);

        if (!dataResponse) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
        }

        let dataNew = [];
        if (dataResponse?.dataArr?.length > 0) {
            for (const tran of dataResponse.dataArr) {
                const transactionTimelineDetails = await this.transactionTimelineDetailsRepository.findByFilterQuery({ transactionId: tran.id });
                dataNew.push({
                    ...tran,
                    transactionTimelineDetails: { ...transactionTimelineDetails[0] }
                })
            }
        }
        

        const data = {
            page: dataResponse.page,
            limit: dataResponse.limit,
            pageCount: dataResponse.pageCount,
            totalDataFound: dataResponse.count,
            data: dataNew,
        };

        return ResponseUtils.successResponseHandler(
            200,
            'Transaction fetched successfully!',
            'transaction',
            data,
        );
    }


    async update(id: string, dto: UpdateTransactionDto, user: any): Promise<TransactionEntity | Error> {
        dto.userId = user.id;
        dto.userName = user.firstName + " " + user.lastName;
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        const tran = await this.repository.findOneByFilterQuery({ id: id })
        // create logs
        const createLogsDto = {
            userId: tran[0].userId,
            address: tran[0].streetAddress,
            clientName: tran[0].clientName,
            informationChnage: "Updated transaction status",
            reasonForChanging: dto.status == TransactionStatus.PENDING ? dto.pendingStatusChangeReason : dto.status == TransactionStatus.OFFER_ACCEPTED ? dto.offerAcceptedStatusChangeReason : dto.status == TransactionStatus.INSPECTION ? dto.inspectionStatusChangeReason : dto.status == TransactionStatus.APPRAISAL ? dto.apraisalStatusChangeReason : dto.status == TransactionStatus.CLEAR_TO_CLOSE ? dto.clearToCloseStatusChangeReason : dto.status == TransactionStatus.CLOSING ? dto.closingStatusChangeReason : dto.status == TransactionStatus.CLOSED ? dto.closedStatusChangeReason : dto.status == TransactionStatus.REJECTED ? dto.rejectedStatusChangeReason : null,
            status: dto.status
        }
        await this.logsRepository.createEntity(createLogsDto);
        return ResponseUtils.successResponseHandler(200, "Data Updated Successfully", "data", data);
    }

    async findOne(id: string): Promise<TransactionEntity[] | Error> {
        const data = await this.repository.findOneByFilterQuery({ id: id });
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data Fetched Successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        try {
            const tran = await this.repository.findOneByFilterQuery({ id: id })
            // create logs
            const createLogsDto = {
                userId: tran[0].userId,
                address: tran[0].streetAddress,
                clientName: tran[0].clientName,
                informationChnage: "Delete a transaction",
                reasonForChanging: null,
                status: null
            }
            await this.logsRepository.createEntity(createLogsDto);

            const timeline: any = await this.transactionTimelineDetailsRepository.findByFilterQuery({ transactionId: id });
            if (timeline) {
                await this.transactionTimelineDetailsRepository.deleteEntity(timeline[0]?.id);
            }

            const notes: any = await this.transactionNotesRepository.findByFilterQuery({ transactionId: id });
            if (notes) {
                await this.transactionTimelineDetailsRepository.deleteEntity(notes[0]?.id);
            }

            const document: any = await this.transactionDocumentsRepository.findByFilterQuery({ transactionId: id });
            if (document) {
                await this.transactionTimelineDetailsRepository.deleteEntity(document[0]?.id);
            }

            const party: any = await this.partyRepository.findByFilterQuery({ transactionId: id });
            if (party) {
                await this.transactionTimelineDetailsRepository.deleteEntity(party[0]?.id);
            }

            const event: any = await this.eventScheduleRepository.findByFilterQuery({ transactionId: id });
            if (event) {
                await this.transactionTimelineDetailsRepository.deleteEntity(event[0]?.id);
            }

            const data = await this.repository.deleteEntity(id);
            if (!data) {
                throw new HttpException(
                    Constants.NOT_FOUND,
                    HttpStatus.NOT_FOUND
                );
            }

            return ResponseUtils.buildDeletedData(data, 200, "Data Deleted Successfully");
        } catch (error) {
            console.log(error);
            return error;
        }
    }
}
